@@ -1,11 +1,11 @@
 /***************************************
-  $Header: /home/amb/procmeter/RCS/procmeter.c 2.25 1998/02/02 19:52:12 amb Exp $
+  $Header: /home/amb/procmeter/RCS/procmeter.c 2.29 1999/03/06 14:42:03 amb Exp $
 
-  ProcMeter - A performance metering/logging program for Linux - Version 2.4b.
+  ProcMeter - A performance metering/logging program for Linux - Version 2.5.1.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1994,95,96,97,98 Andrew M. Bishop
+  This file Copyright 1994,95,96,97,98,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -48,7 +48,10 @@ static char **datfiles;
 /*+ The name of the logfile if any. +*/
 extern char *logfile;
 
-#ifndef NO_X
+#ifdef NO_X
+/*+ A flag that is never set to true. +*/
+int quit=0;
+#else
 /*+ A flag that is set to true when we are told to quit. +*/
 extern int quit;
 
@@ -220,9 +223,9 @@ static void Usage(int verbose)
 
  if(verbose)
    {
-    fprintf(stderr,"\nProcMeter Version 2.4b\n\n");
+    fprintf(stderr,"\nProcMeter Version 2.5.1\n\n");
     fprintf(stderr,"A performance metering/logging program for Linux using /proc.\n");
-    fprintf(stderr,"(c) Andrew Bishop 1994,95,96,97,98. [amb@gedanken.demon.co.uk]\n\n");
+    fprintf(stderr,"(c) Andrew Bishop 1994,95,96,97,98,99. [amb@gedanken.demon.co.uk]\n\n");
    }
 
 #ifdef NO_X
