@@ -1,11 +1,11 @@
 /***************************************
-  $Header: /home/amb/procmeter/RCS/procmeter.h 2.14 1998/01/06 19:39:04 amb Exp $
+  $Header: /home/amb/procmeter/RCS/procmeter.h 2.17 1999/02/24 19:51:29 amb Exp $
 
-  ProcMeter - A performance metering/logging program for Linux - Version 2.4.
+  ProcMeter - A performance metering/logging program for Linux - Version 2.5.1.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1994,95,96,97,98 Andrew M. Bishop
+  This file Copyright 1994,95,96,97,98,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -24,6 +24,7 @@ typedef enum { CPU,
                CPU_IDLE,
                LOAD,
                PROC,
+               FORK,
                CONTEXT,
                SWAP,
                SWAP_IN,
@@ -39,15 +40,23 @@ typedef enum { CPU,
                MEM_USED,
                MEM_BUFF,
                MEM_CACHE,
+               MEM_AVAIL,
                MEM_SWAP,
                LPKT,
+               LBYTE,
                FPKT,
+               FBYTE,
                FPKT_RX,
+               FBYTE_RX,
                FPKT_TX,
+               FBYTE_TX,
                FPKT_COLL,
                SPKT,
+               SBYTE,
                SPKT_RX,
+               SBYTE_RX,
                SPKT_TX,
+               SBYTE_TX,
                BATT_LIFE,
                BATT_LVL,
                NProcStats
